# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy

from . import models as _models
from ._configuration import AzureRedHatOpenShiftClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    MachinePoolsOperations,
    OpenShiftClustersOperations,
    OpenShiftVersionsOperations,
    Operations,
    SecretsOperations,
    SyncIdentityProvidersOperations,
    SyncSetsOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class AzureRedHatOpenShiftClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Rest API for Azure Red Hat OpenShift 4.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.redhatopenshift.operations.Operations
    :ivar open_shift_versions: OpenShiftVersionsOperations operations
    :vartype open_shift_versions: azure.mgmt.redhatopenshift.operations.OpenShiftVersionsOperations
    :ivar open_shift_clusters: OpenShiftClustersOperations operations
    :vartype open_shift_clusters: azure.mgmt.redhatopenshift.operations.OpenShiftClustersOperations
    :ivar machine_pools: MachinePoolsOperations operations
    :vartype machine_pools: azure.mgmt.redhatopenshift.operations.MachinePoolsOperations
    :ivar secrets: SecretsOperations operations
    :vartype secrets: azure.mgmt.redhatopenshift.operations.SecretsOperations
    :ivar sync_identity_providers: SyncIdentityProvidersOperations operations
    :vartype sync_identity_providers:
     azure.mgmt.redhatopenshift.operations.SyncIdentityProvidersOperations
    :ivar sync_sets: SyncSetsOperations operations
    :vartype sync_sets: azure.mgmt.redhatopenshift.operations.SyncSetsOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2023-11-22". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = AzureRedHatOpenShiftClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.open_shift_versions = OpenShiftVersionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.open_shift_clusters = OpenShiftClustersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.machine_pools = MachinePoolsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.secrets = SecretsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.sync_identity_providers = SyncIdentityProvidersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.sync_sets = SyncSetsOperations(self._client, self._config, self._serialize, self._deserialize)

    def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)
